//--------------------------------------------------------------------
//
// Copyright 2002 by Eric Obermuhlner
//
// $Id: template.cpp,v 1.2 2004/04/28 15:00:38 eobermuhlner Exp eobermuhlner $
//
// $Log: template.cpp,v $
// Revision 1.2  2004/04/28 15:00:38  eobermuhlner
// ready for 0.9.5
//
// Revision 1.1  2002/10/26 18:02:47  eric
// Initial revision
//
//
//---------------------------------------------------------------------

extern "C" {
#include "error.h"
#include "template.h"
}

//---------------------------------------------------------------------

class Template
{
public:
  virtual void feed(unsigned char v) = 0;
  virtual bool finished() const = 0;
};

//---------------------------------------------------------------------

class ValueTemplate : public Template
{
public:
  ValueTemplate(int size=1, int base=16);
  
  void feed(unsigned char v);
  bool finished() const;
protected:
  int itsSize;
  int itsBase;
  int itsCount;
};

ValueTemplate::ValueTemplate(int size, int base)
{
  itsSize = size;
  itsBase = base;
  itsCount = 0;
}

void ValueTemplate::feed(unsigned char v)
{
  itsCount++;
  print_text("%02x ", v);
}

bool ValueTemplate::finished() const
{
  return itsCount >= itsSize;
}

//---------------------------------------------------------------------

class TemplateSequence : public Template
{
public:
  TemplateSequence();
  
  virtual void feed(unsigned char v);
  virtual bool finished() const;

  void add(Template* templ);
  void reset();

protected:
  void grow(int count);

protected:
  Template** itsTemplates;
  int itsCount;
  int itsMaxCount;

  int itsPos;
};

TemplateSequence::TemplateSequence()
{
  itsTemplates = 0;
  itsCount = 0;
  itsMaxCount = 0;
  grow(100);

  itsPos = 0;
}

void TemplateSequence::feed(unsigned char v)
{
  itsTemplates[itsPos]->feed(v);

  if(itsTemplates[itsPos]->finished()) {
    itsPos++;
  }
}

bool TemplateSequence::finished() const
{
  return itsPos > itsCount;
}

void TemplateSequence::reset()
{
  itsPos = 0;
}

void TemplateSequence::add(Template* templ)
{
  if(itsCount >= itsMaxCount) {
    grow(itsMaxCount * 2);
  }
  itsTemplates[itsCount++] = templ;
}

void TemplateSequence::grow(int count)
{
  Template **t = new Template*[count];

  if(itsTemplates) {
    int i;
    for(i=0 ; i<itsCount ; i++) {
      t[i] = itsTemplates[i];
    }
    delete itsTemplates;
  }

  itsMaxCount = count;
}

//---------------------------------------------------------------------

//extern "C" {

static Template* staticTemplate = 0;

void init_template(const char* templateName)
{
  staticTemplate = new ValueTemplate();
}

void feed_template(unsigned char v)
{
  staticTemplate->feed(v);
}

//}

//---------------------------------------------------------------------
